//   rdhpiplaystream.h
//
//   A class for playing Microsoft WAV files.
//
//   (C) Copyright 2002-2007 Fred Gleason <fredg@paravelsystems.com>
//
//    $Id: rdhpiplaystream.h,v 1.1 2007/09/14 14:06:53 fredg Exp $
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//

#ifndef RDHPIPLAYSTREAM_H
#define RDHPIPLAYSTREAM_H

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <qobject.h>
#include <qwidget.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qdatetime.h>
#include <rdwavefile.h>
#include <rdhpisoundcard.h>

#include <asihpi/hpi.h>
#define MAX_FRAGMENT_SIZE 192000
#define FRAGMENT_TIME 50
#define TIMESCALE_LOW_LIMIT 800
#define TIMESCALE_HIGH_LIMIT 1200
//#define RPLAYSTREAM_SHOW_SLOTS yes


class RDHPIPlayStream : public QObject,public RDWaveFile
{
  Q_OBJECT

 public:
  enum State {Stopped=0,Playing=1,Paused=2};
  enum Error {Ok=0,NoFile=1,NoStream=2,AlreadyOpen=3};
  RDHPIPlayStream(RDHPISoundCard *card,QWidget *parent=0,const char *name=0);
  ~RDHPIPlayStream();
  QString errorString(RDHPIPlayStream::Error err);
  bool formatSupported(RDWaveFile::Format format);
  bool formatSupported();
  RDHPIPlayStream::Error openWave();
  RDHPIPlayStream::Error openWave(QString filename);
  void closeWave();
  int getCard() const;
  int getStream() const;
  int getSpeed() const;
  bool setSpeed(int speed,bool pitch=false,bool rate=false);
  RDHPIPlayStream::State getState() const;
  
 signals:
  void isStopped(bool state);
  void played();
  void paused();
  void stopped();
  void position(int samples);
  void stateChanged(int card,int stream,int state);

 public slots:
  void setCard(int card);
  bool play();
  void pause();
  void stop();
  int currentPosition();
  bool setPosition(unsigned samples);
  void setPlayLength(int length);
  void tickClock();
  
 private:
  void Drained();
  int GetStream();
  void FreeStream();
  RDHPISoundCard *sound_card;
  RDHPIPlayStream::State stream_state;
  QString wave_name;
  QTimer *clock;
  int card_number;
  int stream_number;
  bool is_open;
  bool playing;
  bool is_paused;
  bool repositioned;
  bool stopping;
  unsigned audio_ptr;
  unsigned left_to_write;
  unsigned read_bytes;
  unsigned samples_skipped;
  int play_length;
  QTimer *play_timer;
  QTime start_time;
  int play_speed;
  bool pitch_can_vary;
  bool rate_can_vary;
  HPI_HOSTREAM hpi_stream;
  HW16 state;
  HW32 buffer_size;
  HW32 data_to_play;
  HW32 samples_played;
  HW32 reserved;
  int fragment_time;
  HW8 *pdata;
  HPI_FORMAT format;
#if HPI_VER < 0x00030500
  HPI_DATA hpi_data;
#endif
  HW32 fragment_size;
  bool restart_transport;
  int samples_pending;
  unsigned current_position;
};


#endif  // RDHPIPLAYSTREAM_H
